@@ -1,5 +1,5 @@
 /* ltdl.c -- system independent dlopen wrapper
-   Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999, 2000, 2008, 2009 Free Software Foundation, Inc.
    Originally by Thomas Tanner <tanner@ffii.org>
    This file is part of GNU Libtool.
 
@@ -1312,6 +1312,7 @@ sys_wll_open (loader_data, filename)
   char	       *searchname = 0;
   char	       *ext;
   char		self_name_buf[MAX_PATH];
+  UINT errormode;
 
   if (!filename)
     {
@@ -1341,6 +1342,12 @@ sys_wll_open (loader_data, filename)
   if (!searchname)
     return 0;
 
+  /* Silence dialog from LoadLibrary on some failures.
+     No way to get the error mode, but to set it,
+     so set it twice to preserve any previous flags. */
+  errormode = SetErrorMode (SEM_FAILCRITICALERRORS);
+  SetErrorMode (errormode | SEM_FAILCRITICALERRORS);
+
 #if __CYGWIN__
   {
     char wpath[MAX_PATH];
@@ -1350,6 +1357,7 @@ sys_wll_open (loader_data, filename)
 #else
   module = LoadLibrary (searchname);
 #endif
+  SetErrorMode (errormode);
   LT_DLFREE (searchname);
 
   /* libltdl expects this function to fail if it is unable
@@ -2164,7 +2172,8 @@ static	int	trim		      LT_PARAMS((char **dest,
 static	int	try_dlopen	      LT_PARAMS((lt_dlhandle *handle,
 						 const char *filename));
 static	int	tryall_dlopen	      LT_PARAMS((lt_dlhandle *handle,
-						 const char *filename));
+						 const char *filename,
+						 const char * useloader));
 static	int	unload_deplibs	      LT_PARAMS((lt_dlhandle handle));
 static	int	lt_argz_insert	      LT_PARAMS((char **pargz,
 						 size_t *pargz_len,
@@ -2350,9 +2359,10 @@ lt_dlexit ()
 }
 
 static int
-tryall_dlopen (handle, filename)
+tryall_dlopen (handle, filename, useloader)
      lt_dlhandle *handle;
      const char *filename;
+     const char *useloader;
 {
   lt_dlhandle	 cur;
   lt_dlloader   *loader;
@@ -2419,6 +2429,11 @@ tryall_dlopen (handle, filename)
 
   while (loader)
     {
+      if (useloader && strcmp(loader->loader_name, useloader))
+	{
+	  loader = loader->next;
+	  continue;
+	}
       lt_user_data data = loader->dlloader_data;
 
       cur->module = loader->module_open (data, filename);
@@ -2488,7 +2503,7 @@ tryall_dlopen_module (handle, prefix, dirname, dlname)
       error += tryall_dlopen_module (handle,
 				     (const char *) 0, prefix, filename);
     }
-  else if (tryall_dlopen (handle, filename) != 0)
+  else if (tryall_dlopen (handle, filename, NULL) != 0)
     {
       ++error;
     }
@@ -2509,7 +2524,7 @@ find_module (handle, dir, libdir, dlname, old_name, installed)
   /* Try to open the old library first; if it was dlpreopened,
      we want the preopened version of it, even if a dlopenable
      module is available.  */
-  if (old_name && tryall_dlopen (handle, old_name) == 0)
+  if (old_name && tryall_dlopen (handle, old_name, "dlpreload") == 0)
     {
       return 0;
     }
@@ -2773,7 +2788,7 @@ find_handle_callback (filename, data, ignored)
 
   /* Try to dlopen the file, but do not continue searching in any
      case.  */
-  if (tryall_dlopen (handle, filename) != 0)
+  if (tryall_dlopen (handle, filename,NULL) != 0)
     *handle = 0;
 
   return 1;
@@ -3058,7 +3073,7 @@ try_dlopen (phandle, filename)
       /* lt_dlclose()ing yourself is very bad!  Disallow it.  */
       LT_DLSET_FLAG (*phandle, LT_DLRESIDENT_FLAG);
 
-      if (tryall_dlopen (&newhandle, 0) != 0)
+      if (tryall_dlopen (&newhandle, 0, NULL) != 0)
 	{
 	  LT_DLFREE (*phandle);
 	  return 1;
@@ -3174,13 +3189,13 @@ try_dlopen (phandle, filename)
 	    }
 #endif
 #ifdef LTDL_SYSSEARCHPATH
-	  if (!file && sys_search_path)
+	  if (!file)
 	    {
 	      file = find_file (sys_search_path, base_name, &dir);
 	    }
 #endif
 	}
-      if (!file)
+      else
 	{
 	  file = fopen (filename, LT_READTEXT_MODE);
 	}
@@ -3216,6 +3231,13 @@ try_dlopen (phandle, filename)
 	  while ((line[LT_STRLEN(line) -1] != '\n') && (!feof (file)))
 	    {
 	      line = LT_DLREALLOC (char, line, line_len *2);
+	      if (!line)
+                {
+                  fclose (file);
+                  ++errors;
+                  goto cleanup;
+                }
+
 	      if (!fgets (&line[line_len -1], (int) line_len +1, file))
 		{
 		  break;
@@ -3364,7 +3386,7 @@ try_dlopen (phandle, filename)
 #endif
 		   )))
 	{
-          if (tryall_dlopen (&newhandle, filename) != 0)
+          if (tryall_dlopen (&newhandle, filename, NULL) != 0)
             {
               newhandle = NULL;
             }
@@ -3564,6 +3586,7 @@ lt_argz_insertdir (pargz, pargz_len, dirnam, dp)
   assert (pargz);
   assert (pargz_len);
   assert (dp);
+  assert (dirnam);
 
   dir_len = LT_STRLEN (dirnam);
   end     = dp->d_name + LT_D_NAMLEN(dp);
