@@ -12,7 +12,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006,2008
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006,2008,2009
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -57,6 +57,7 @@
  ***********************************************************************/
 
 #include "gstpriv.h"
+#include "gstpub.h"
 
 #ifndef NAN
 #define NAN (0.0 / 0.0)
@@ -104,7 +105,8 @@ _gst_va_msg_send (OOP receiver,
   save = ap;
 #endif
 
-  for (numArgs = 0; (anArg = va_arg (ap, OOP)) != NULL; numArgs++);
+  for (numArgs = 0; va_arg (ap, OOP) != NULL; numArgs++)
+    ;
 
   if (numArgs != _gst_selector_num_args (selector))
     return (_gst_nil_oop);
@@ -206,7 +208,7 @@ _gst_va_msg_sendf (PTR resultPtr,
     return;
 
   args = (OOP *) alloca (sizeof (OOP) * numArgs);
-  for (i = -1, s = selectorBuf, fp = &fmt[2]; *fp; fp++)
+  for (i = -1, fp = &fmt[2]; *fp; fp++)
     {
       if (*fp != '%')
 	continue;
@@ -1104,7 +1106,7 @@ _gst_copy_registered_oops (void)
 	 lies.  */
       OOP *first = *(k->first);
       OOP *last = *(k->last);
-      COPY_OOP_RANGE (first, last);
+      _gst_copy_oop_range (first, last);
     }
 }
 
@@ -1132,6 +1134,50 @@ _gst_mark_registered_oops (void)
 	 lies.  */
       OOP *first = *(k->first);
       OOP *last = *(k->last);
-      MARK_OOP_RANGE (first, last);
+      _gst_mark_oop_range (first, last);
     }
 }
+
+void
+_gst_init_vmproxy (void)
+{
+  gst_interpreter_proxy.nilOOP = _gst_nil_oop;
+  gst_interpreter_proxy.trueOOP = _gst_true_oop;
+  gst_interpreter_proxy.falseOOP = _gst_false_oop;
+
+  gst_interpreter_proxy.objectClass = _gst_object_class;
+  gst_interpreter_proxy.arrayClass = _gst_array_class;
+  gst_interpreter_proxy.stringClass = _gst_string_class;
+  gst_interpreter_proxy.characterClass = _gst_char_class;
+  gst_interpreter_proxy.smallIntegerClass = _gst_small_integer_class;
+  gst_interpreter_proxy.floatDClass = _gst_floatd_class;
+  gst_interpreter_proxy.floatEClass = _gst_floate_class;
+  gst_interpreter_proxy.byteArrayClass = _gst_byte_array_class;
+  gst_interpreter_proxy.objectMemoryClass = _gst_object_memory_class;
+  gst_interpreter_proxy.classClass = _gst_class_class;
+  gst_interpreter_proxy.behaviorClass = _gst_behavior_class;
+  gst_interpreter_proxy.blockClosureClass = _gst_block_closure_class;
+  gst_interpreter_proxy.contextPartClass = _gst_context_part_class;
+  gst_interpreter_proxy.blockContextClass = _gst_block_context_class;
+  gst_interpreter_proxy.methodContextClass = _gst_method_context_class;
+  gst_interpreter_proxy.compiledMethodClass = _gst_compiled_method_class;
+  gst_interpreter_proxy.compiledBlockClass = _gst_compiled_block_class;
+  gst_interpreter_proxy.fileDescriptorClass = _gst_file_descriptor_class;
+  gst_interpreter_proxy.fileStreamClass = _gst_file_stream_class;
+  gst_interpreter_proxy.processClass = _gst_process_class;
+  gst_interpreter_proxy.semaphoreClass = _gst_semaphore_class;
+  gst_interpreter_proxy.cObjectClass = _gst_c_object_class;
+
+  /* And system objects.  */
+  gst_interpreter_proxy.processorOOP = _gst_processor_oop;
+}
+
+struct VMProxy *
+_gst_get_vmproxy (void)
+{
+  struct VMProxy *result;
+
+  result = xmalloc (sizeof (struct VMProxy));
+  memcpy (result, &gst_interpreter_proxy, sizeof (struct VMProxy));
+  return result;
+}
