@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006,2008
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006,2008,2009
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -69,9 +69,9 @@
 #include <math.h>
 #include <float.h>
 #include <errno.h>
+#include <signal.h>
 #include <sys/stat.h>
 #include <limits.h>
-#include <poll.h>
 #include <ctype.h>
 #include <wchar.h>
 #include <dirent.h>
@@ -361,15 +361,15 @@ enum {
 
 /* Answer whether a method, OOP, has already been verified. */
 #define IS_OOP_VERIFIED(oop) \
-  ((oop)->flags & F_VERIFIED)
+  (((oop)->flags & F_VERIFIED) != 0)
 
 /* Answer whether an object, OOP, is weak.  */
 #define IS_OOP_WEAK(oop) \
-  ((oop)->flags & F_WEAK)
+  (((oop)->flags & F_WEAK) != 0)
 
 /* Answer whether an object, OOP, is readonly.  */
 #define IS_OOP_READONLY(oop) \
-  (IS_INT ((oop)) ? F_READONLY : (oop)->flags & F_READONLY)
+  (IS_INT ((oop)) || ((oop)->flags & F_READONLY))
 
 /* Set whether an object, OOP, is readonly or readwrite.  */
 #define MAKE_OOP_READONLY(oop, ro) \
@@ -380,7 +380,7 @@ enum {
 
 /* Answer whether an object, OOP, is untrusted.  */
 #define IS_OOP_UNTRUSTED(oop) \
-  (IS_INT ((oop)) ? 0 : ((oop)->flags & F_UNTRUSTED))
+  (!IS_INT ((oop)) && ((oop)->flags & F_UNTRUSTED))
 
 /* Set whether an object, OOP, is trusted or untrusted.  */
 #define MAKE_OOP_UNTRUSTED(oop, untr) \
@@ -515,6 +515,12 @@ extern OOP _gst_nil_oop
 #define BYTES_TO_SIZE(bytes) \
   ((bytes) / sizeof (PTR))
 
+#ifdef __GNUC__
+#define no_opt(x)	({ __typeof__ ((x)) _result; \
+			 asm ("" : "=r" (_result) : "0" ((x))); _result; })
+#else
+#define no_opt(x)	(x)
+#endif
 
 /* integer conversions and some information on SmallIntegers.  */
 
@@ -579,13 +585,11 @@ extern OOP _gst_nil_oop
 #endif
 
 #include "ansidecl.h"
-#include "signalx.h"
 #include "mathl.h"
 #include "socketx.h"
 #include "strspell.h"
 #include "alloc.h"
 #include "md-config.h"
-#include "memzero.h"
 #include "avltrees.h"
 #include "rbtrees.h"
 
@@ -612,6 +616,8 @@ extern OOP _gst_nil_oop
 #include "mpz.h"
 #include "print.h"
 #include "security.h"
+#include "real.h"
+#include "sockets.h"
 
 /* Include this last, it has the bad habit of #defining printf
    and this fools gcc's __attribute__ (format) */
